//
//	I2CLCDhCo (ACM1602p)
//		copyright (c) 2012,2013 T.Nakao    http://www.wsnak.com
//
//		WireƕpKv
//

#ifndef wI2cLcd_h
#define wI2cLcd_h

#include <inttypes.h>
#include "Print.h"


// commands
#define LCD_CLEARDISPLAY 0x01
#define LCD_RETURNHOME 0x02
#define LCD_ENTRYMODESET 0x04
#define LCD_DISPLAYCONTROL 0x08
#define LCD_CURSORSHIFT 0x10
#define LCD_FUNCTIONSET 0x20
#define LCD_SETCGRAMADDR 0x40
#define LCD_SETDDRAMADDR 0x80

// flags for display entry mode
#define LCD_ENTRYRIGHT 0x00
#define LCD_ENTRYLEFT 0x02
#define LCD_ENTRYSHIFTINCREMENT 0x01
#define LCD_ENTRYSHIFTDECREMENT 0x00

// flags for display on/off control
#define LCD_DISPLAYON 0x04
#define LCD_DISPLAYOFF 0x00
#define LCD_CURSORON 0x02
#define LCD_CURSOROFF 0x00
#define LCD_BLINKON 0x01
#define LCD_BLINKOFF 0x00

// flags for display/cursor shift
#define LCD_DISPLAYMOVE 0x08
#define LCD_CURSORMOVE 0x00
#define LCD_MOVERIGHT 0x04
#define LCD_MOVELEFT 0x00

// flags for function set
#define LCD_8BITMODE 0x10
#define LCD_4BITMODE 0x00
#define LCD_2LINE 0x08
#define LCD_1LINE 0x00
#define LCD_5x10DOTS 0x04
#define LCD_5x8DOTS 0x00


class wI2cLcd : public Print {
public:
	wI2cLcd(uint8_t bm8 = LCD_8BITMODE);

	void init(void);
	void begin(uint8_t cols, uint8_t lines, uint8_t adrs = 0x50);

	void clear(void);
	void home(void);
	void setCursor(uint8_t col, uint8_t row);

	void noDisplay(void);
	void display(void);
	void noBlink(void);
	void blink(void);
	void noCursor(void);
	void cursor(void);
	void scrollDisplayLeft(void);
	void scrollDisplayRight(void);
	void leftToRight(void);
	void rightToLeft(void);
	void autoscroll(void);
	void noAutoscroll(void);

	void createChar(uint8_t, uint8_t[] );

	void writeCmd(uint8_t cmd);
	void writeData(uint8_t dat);

	virtual size_t write(uint8_t);

	using Print::write;

	void setCursor(uint8_t col, uint8_t row, uint8_t area);	// 40x4 120509
	void selectArea(uint8_t sw);		// 40x4  120509

	uint8_t slaveAddress;		// I2C slave address 120509
	uint8_t initFlag;
	uint8_t currArea;		// 40x4 120509

private:
	uint8_t	_numlines;
	uint8_t _displayfunction;
	uint8_t _displaymode;
	uint8_t _displaycontrol;

	uint8_t _bitmode;
	uint8_t _w_area;
};

#endif
